@@ -3,13 +3,14 @@ import json
 import uuid
 import hashlib
 import base64
+import time
 import traceback
 
+from typing import List
+
 from azure.storage.queue import QueueClient
 from azure.storage.blob import BlobClient
 from azure.cosmosdb.table import TableService, TableBatch
-from typing import List
-from multiprocessing.pool import ThreadPool
 from azure.core.exceptions import ResourceExistsError, ResourceNotFoundError
 
 from .common import TaskResult, TaskMetadata, chunks
@@ -221,6 +222,16 @@ class MessageQueueClient:
             except ResourceExistsError:
                 pass
 
+    def receive_message(self, timeout: int) -> str:
+        message = self.queue.receive_message(visibility_timeout=timeout, timeout=timeout)
+        if message is None:
+            return None, None
+        body = base64.b64decode(message.content).decode()
+        return body, message
+
+    def delete_message(self, message):
+        self.queue.delete_message(message.id, message.pop_receipt)
+
     def enqueue(self, request_id):
         if type(request_id) is str:
             self.queue.send_message(base64.b64encode(request_id.encode()).decode())
@@ -236,25 +247,16 @@ class MessageQueueClient:
                 self.poison_queue.delete_message(message)
 
 
-class Worker:
-    def __init__(self, worker_name: str, 
-        create_if_not_exist=True, 
-        segment_limit=200, 
-        connection_string=None, 
-        return_only_source=True
-    ):
+class WorkerConnector:
+    def __init__(self, worker_name: str, connection_string=None):
         if connection_string is None:
             self.connection_string = os.environ['AzureWebJobsStorage']
         else:
             self.connection_string = connection_string
         self.worker_name = worker_name
-        self.segment_limit = segment_limit
-
-        self.reader = BlobReader(connection_string, return_only_source)
-        self.metadata_repository = MetadataRepository(connection_string, worker_name + "Data", create_if_not_exist)
-        self.message_queue_client = MessageQueueClient(worker_name.lower(), worker_name.lower() + "-poison", connection_string, create_if_not_exist)
-        self.pool = ThreadPool(processes=1)
-
+        self.metadata_repository = MetadataRepository(self.connection_string, worker_name + "Data", False)
+        self.message_queue_client = MessageQueueClient(worker_name.lower(), worker_name.lower() + "-poison", self.connection_string, False)
+    
     def exists(self) -> bool:
         return self.metadata_repository.exists()
 
@@ -286,6 +288,42 @@ class Worker:
             entity.result = self._expand_result(entity.result)
         return entities
 
+    def _expand_result(self, result):
+        if result is None:
+            return None
+
+        # blob path: "{worker}/Requests/{pk}/{rk}"
+        if len(result.split("/")) == 4:
+            return self.reader.read_output_blob(result)
+        return result
+
+
+class Worker:
+    def __init__(self, worker_name: str, 
+        segment_limit=200, 
+        connection_string=None, 
+        return_only_source=True
+    ):
+        if connection_string is None:
+            self.connection_string = os.environ['AzureWebJobsStorage']
+        else:
+            self.connection_string = connection_string
+        self.worker_name = worker_name
+        self.segment_limit = segment_limit
+        self.reader = BlobReader(self.connection_string, return_only_source)
+        self.metadata_repository = MetadataRepository(self.connection_string, worker_name + "Data", True)
+        self.message_queue_client = MessageQueueClient(worker_name.lower(), worker_name.lower() + "-poison", self.connection_string, True)
+
+    def process_messages_from_queue(self, process_func, timeout=3600, sleep_timeout=1):
+        while True:
+            time.sleep(sleep_timeout)
+            message = self.message_queue_client.receive_message(timeout)
+            if message is None:
+                continue
+            body, message = base64.b64decode(message.content).decode()
+            self.process_message(body, process_func)
+            self.message_queue_client.delete_message(message)
+
     def process_message(self, request_id: str, func):    
         metadata = self.metadata_repository.get_task_metadata(request_id)
         try:
@@ -306,15 +344,6 @@ class Worker:
         except ResourceNotFoundError:
             pass
 
-    def _expand_result(self, result):
-        if result is None:
-            return None
-
-        # blob path: "{worker}/Requests/{pk}/{rk}"
-        if len(result.split("/")) == 4:
-            return self.reader.read_output_blob(result)
-        return result
-
     def _process_raw_request(self, metadata: TaskMetadata, func):
         request_body = json.loads(metadata.request)
         response = func(request_body["Data"], request_body["Params"])
